/*                                                           05-31-95 07:40pm
 ****************************************************************************
 *   DBLIB   Version 1.10.02    dBASE-file C Interface Library.
 ****************************************************************************
 *
 *   DBLIB.H          Header File of dBASE-file C Interface Library.
 *
 * Copyright (c) Alexander V. Loukyanets, 1994,95.  All rights reserved.
 *
 ****************************************************************************
 *
 * Compiling Notes:
 *   File(s) included this header file must be compiling with byte alignment
 *   of struct members.
 */


#ifndef _DBFlib_H_Module
#define _DBFlib_H_Module  1


/*
 ****************************************************************************
 *   Constant definitions
 */

#define   DB2FILE             2
#define   DB3FILE             3
#define   DB3WITHMEMO         0x83
#define   DBFIELD_REC_LEN     32      //  length of field description record
#define   DBHEADER_PROLOG     32      //  length of header without field desc and terminator
#define   DBMAX_HEADER        4129    //  maximum length of dBase III header
#define   DBMAX_RECORD        4000    //  dBase III record limit
#define   DBMAX_FIELDS        128     //  dBase III field limit
#define   DBMAX_MEMO_BYTES    512     //  dBase III memo field record size

#define   MAXPATH             80

/*** error codes ***/
#define   DBE_OK              0       //  it's OK
#define   DBE_OUT_OF_MEM      8       //  insufficient memory error
#define   DBE_NO_FILE         2       //  file not found error
#define   DBE_BAD_FORMAT      11      //  file not dBASE III file
#define   DBE_RECNO_TOO_BIG   105     //  requested record too big


/*
 ****************************************************************************
 *   Types definitions
 */

typedef struct FIELD_RECORD         // This structure is filled in memory
{                                   // with a fread.  Do not change.
    char            name[11];           // name of field in asciz
    char            typ;                // type of field...char,numeric etc.
    char           *field_data_address; // offset of field in record
#if defined(__TINY__) || defined(__SMALL__) || defined (__MEDIUM__)
    int             space_holder;       // field_data_address must be 32 bits
#endif
    unsigned char   len;                // length of field
    unsigned char   dec;                // decimals in field
    unsigned char   reserved_bytes[14]; // reserved by dbase
}            t_FIELD_RECORD;

typedef struct DBF
{
    char            filename[MAXPATH];  // dos filename
    FILE           *file_ptr;           // c file pointer
    unsigned long   current_record;     // current record in memory
    enum                                // status of file
    {
      db_not_open=0,
      db_not_updated,
      db_updated
    }               status;
    enum                                // Opening method
    {
      db_via_db_open=0,                 // - via function db_open()
      db_via_db_cpystr                  // - via function db_cpystr()
    }               met_opening;
    unsigned char   num_fields;         // number of fields

    // the following 7 variables are filled with a fread, do not change order or size:
    unsigned char   dbf_version;        // version character
    unsigned char   update_yr;          // date of last update - year (-1900)
    unsigned char   update_mo;          // date of last update - month
    unsigned char   update_day;         // date of last update - day
    unsigned long   records;            // number of records in dbf
    unsigned int    header_length;      // length of header structure
    unsigned int    record_length;      // length of a record

    t_FIELD_RECORD *fields_ptr;         // pointer to field array
    char           *record_ptr;         // pointer to current record struct
}            t_DBF;


/*
 ****************************************************************************
 *   Prototypes
 */

int   db_addrec   ( struct DBF *d );
int   db_blank    ( struct DBF *d );
int   db_close    ( struct DBF *d );
int   db_cpystr   ( struct DBF *s, struct DBF *d );
char  db_getfld   ( struct DBF *d, int f, char *buff );
int   db_getrec   ( struct DBF *d, unsigned long int r );
int   db_open     ( struct DBF *d );
int   db_putfld   ( struct DBF *d, int f, char *buff );
int   db_putrec   ( struct DBF *d, unsigned long int r );


#endif
/****************************************************************************
*** end of DBLIB.H ***/