/*                                                           06-25-95 12:54pm
 ****************************************************************************
 *   DBLIB   Version 1.10.02    dBASE-file C Interface Library.
 ****************************************************************************
 *
 *   DB_CLOSE.C       Close a dBASE file after access by the other routines
 * ( db_close )       in DBLIB. Updates header and places EOF marker at end
 *                    of file.
 *
 * Copyright (c) Alexander V. Loukyanets, 1994,95.  All rights reserved.
 *
 ****************************************************************************
 *
 * Usage:
 *      d = (struct DBF *)malloc( sizeof( struct DBF ) );
 *      strcpy( d->filename, "filename.dbf" );
 *      db_open( d );
 *      ... access file with other routines ...
 *      db_close( d );
 *      free( d );
 *
 * Notes:
 *      ALWAYS close a file that has been opened, otherwise records may be
 *      lost.
 *
 ****************************************************************************
 */

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include "dblib.h"
#include "dbdeb.h"


int db_close( struct DBF *d )
{
  struct tm  *dt;
  time_t      ltime;

  if ( d->status == db_updated ) {
    //**************************** update date data
    time( &ltime );
    dt = localtime( &ltime );
    d->update_day = dt->tm_mday;
    d->update_mo  = dt->tm_mon+1;
    d->update_yr  = dt->tm_year;

    //**************************** position at start of file
    rewind( d->file_ptr );
    //********************** rewrite header
    fwrite( &d->dbf_version, 1, 12, d->file_ptr );

    if ( d->met_opening == db_via_db_cpystr ) {
      //**************************** position at end of file
      fseek( d->file_ptr, 0L, 2 );
      //********************** write eof
      fwrite( "\x1a", 1, 1, d->file_ptr );
    }
	}

  //**************************** free fields array and record
  FREE( d->fields_ptr );
  FREE( d->record_ptr );
  fclose( d->file_ptr );

  return( DBE_OK );
}


/****************************************************************************
*** end of DB_CLOSE.C ***/