/*                                                           05-31-95 07:40pm
 ****************************************************************************
 *   DBLIB   Version 1.10.02    dBASE-file C Interface Library.
 ****************************************************************************
 *
 *   DB_CPYST.C       Create a new dBASE file useing the structure of an
 * ( db_cpystr )      existing file.
 *
 * Copyright (c) Alexander V. Loukyanets, 1994,95.  All rights reserved.
 *
 ****************************************************************************
 *
 * Usage:
 *      source = (struct DBF *)malloc( sizeof( struct DBF ) );
 *      dest   = (struct DBF *)malloc( sizeof( struct DBF ) );
 *      strcpy( source->filename, "source.dbf" );
 *      strcpy(   dest->filename, "dest.dbf" );
 *      db_open( source );
 *      db_cpystr( source, dest );
 *      db_close( source );
 *      db_close( dest );
 *      free( source );
 *      free( dest );
 *
 * Notes:
 *      This code fragment does the same thing as:
 *        USE SOURCE
 *        COPY STRUCTURE TO DEST
 *
 * Returns:
 *      DBE_OK          if successful with structure filled in
 *      DBE_NO_FILE     if unable to find file
 *      DBE_OUT_OF_MEM  if not enough memory
 *
 ****************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "dblib.h"
#include "dbdeb.h"


int db_cpystr( struct DBF *s, struct DBF *d )
{
  struct FIELD_RECORD  *f;
  register int          i;
  register int          n;

  //*************************** make sure source file is open
  if ( s->status != db_not_updated  &&
       s->status != db_updated )
    return( DBE_NO_FILE );

  d->status = db_not_open;    //*** in case can not open

  if ( NULL == (d->file_ptr = fopen(d->filename,"w+b")) )
    return( DBE_NO_FILE );

  d->met_opening = db_via_db_cpystr;

  if ( NULL == (d->fields_ptr = (struct FIELD_RECORD *)MALLOC( s->num_fields * DBFIELD_REC_LEN )) )
    return( DBE_OUT_OF_MEM );

  memcpy( d->fields_ptr, s->fields_ptr, (s->num_fields * DBFIELD_REC_LEN) );
  memcpy( &d->dbf_version, &s->dbf_version, 12 );
  d->records = 0L;
  d->current_record = 0;
  d->num_fields = s->num_fields;
  //*** the following line forces the copied dbf structure to be in dBASE III+ format
  //*** dBASE III+ files can be read without problems with dBASE III.
  d->header_length = DBHEADER_PROLOG + (d->num_fields * DBFIELD_REC_LEN) + 1;
  if ( NULL == (d->record_ptr = (char *)MALLOC( d->record_length )) )
    return( DBE_OUT_OF_MEM );
  //******************************** initialize pointers to fields in record
  for ( i=0, n=1;  i < d->num_fields;  i++ ) {
    f = d->fields_ptr + i;
    f->field_data_address = d->record_ptr + n;
    n += f->len;
  }
  fwrite( &d->dbf_version, 1, 12, d->file_ptr );
  for ( i=1; i<=20; i++ ) fwrite( "\0", 1, 1, d->file_ptr );
  fwrite( d->fields_ptr, DBFIELD_REC_LEN, d->num_fields, d->file_ptr );
  fwrite( "\x0d", 1, 1, d->file_ptr );
  d->status = db_updated;         //*** open successfull

  return( DBE_OK );
}


/****************************************************************************
*** end of DB_CPYST.C ***/