/*                                                           05-31-95 07:40pm
 ****************************************************************************
 *   DBLIB   Version 1.10.02    dBASE-file C Interface Library.
 ****************************************************************************
 *
 *   DB_GETFL.C       Fill a buffer with field data from a dBASE file
 * ( db_getfld )      opened with db_open and accessed with db_getrec.
 *                    The data from the record is copied into the buffer
 *                    and terminated with '\0'.
 *
 * Copyright (c) Alexander V. Loukyanets, 1994,95.  All rights reserved.
 *
 ****************************************************************************
 *
 * Usage:
 *      d = (struct DBF *)malloc( sizeof( struct DBF ) );
 *      strcpy( d->filename, "filename.dbf" );
 *      db_open( d );
 *      db_getrec( d, (long)recordno );
 *      db_getfld( d, fieldno, buffer );
 *      ... use field data ...
 *      db_close( d );
 *      free( d );
 *
 * Returns:
 *      field type if successful,
 *      null if not.
 *
 ****************************************************************************
 */

#include <stdio.h>
#include <string.h>
#include "dblib.h"
#include "dbdeb.h"


char db_getfld( struct DBF *d, int f, char *buff )
{
  struct FIELD_RECORD  *fp;

  if ( f > 0  &&  f <= d->num_fields ) {
    fp = d->fields_ptr + (f - 1);
    memcpy( buff, fp->field_data_address, fp->len );
    buff[fp->len] = '\0';
    return( fp->typ );
  }
  else {
    buff[0] = '\0';
    return( NULL );
  }
}


/****************************************************************************
*** end of DB_GETFL.C ***/