/*                                                           05-31-95 07:40pm
 ****************************************************************************
 *   DBLIB   Version 1.10.02    dBASE-file C Interface Library.
 ****************************************************************************
 *
 *   DB_OPEN.C        Open a dBASE file for access by the other routines
 * ( db_open )        of DBLIB.
 *
 * Copyright (c) Alexander V. Loukyanets, 1994,95.  All rights reserved.
 *
 ****************************************************************************
 *
 * Usage:
 *      d = (struct DBF *)malloc( sizeof( struct DBF ) );
 *      strcpy( d->filename, "filename.dbf" );
 *      db_open( d );
 *      ... access file with other routines ...
 *      db_close( d );
 *      free( d );
 *
 * Returns:
 *      DBE_OK          if successful with structure filled in
 *      DBE_NO_FILE     if unable to find file
 *      DBE_OUT_OF_MEM  if not enough memory
 *      DBE_BAD_FORMAT  if not dBASE file
 *
 ****************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include "dblib.h"
#include "dbdeb.h"


const char *dblib_Copyright = "\nDBLIB dBASE III C Interface Library Version 1.10.02(31.05.95)"
      #if defined( DB_DEBUG )
                              " <<<DEBUGGING VERSION>>>"
      #endif
                              "\nCopyright (c) 1994,95 Alexander V. Loukyanets, ALL RIGHTS RESERVED.";


int db_open( struct DBF *d )
{
  register int  i;
  register int  n;

  d->status = db_not_open;      //*** in case can not open
  if ( NULL == (d->file_ptr = fopen( d->filename, "r+b" )) )
    return( DBE_NO_FILE );

  d->met_opening = db_via_db_open;

  rewind( d->file_ptr );

  //********************************** read prolog
  fread( (void *)&d->dbf_version, (unsigned)1, (unsigned)12, d->file_ptr);

  if ( d->dbf_version != DB3FILE        //*** check for dbiii file marker
    && d->dbf_version != DB3WITHMEMO
    || d->update_mo == 0 ) {
      fclose( d->file_ptr );
      return( DBE_BAD_FORMAT );
  }

  d->current_record = 0L;
  d->num_fields = ((d->header_length - (DBFIELD_REC_LEN+1)) / DBHEADER_PROLOG);

  if ( NULL == (d->fields_ptr = (struct FIELD_RECORD *)MALLOC( (unsigned)(d->num_fields * DBFIELD_REC_LEN) )) )
    return( DBE_OUT_OF_MEM );

  //****************************** position at field descriptions
  fseek( d->file_ptr, (long)DBHEADER_PROLOG, 0 );

  //****************************** read into field description array
  fread( (void *)d->fields_ptr, sizeof( *d->fields_ptr ), (unsigned)d->num_fields, d->file_ptr );

  if ( NULL == (d->record_ptr = (char *)MALLOC( d->record_length )) )
    return( DBE_OUT_OF_MEM );

  //****************************** initialize pointers to fields in record.
  for ( i=0, n=1;  i < d->num_fields;  i++ ) {
            //*** n is offset from start of rec
            //*** @ n=0 is the deleted record flag
    d->fields_ptr[i].field_data_address = d->record_ptr + n;
    n += d->fields_ptr[i].len;
  }

  d->status = db_not_updated;   //*** open successfull
  return(0);
}


#if defined( DB_DEBUG_MAIN )   /***** test program *****/

main( int argc, char **argv )
{
  struct DBF  d;
  int         errornum;
  int         i;

  if (argc != 2) {
    printf( "Usage db_open filename" );
    exit( 1 );
  }

  strcpy( d.filename, argv[1] );
  if ( !strchr( d.filename, '.' ) )         // default to .dbf file
        strcat( d.filename, ".DBF" );

  if ( (errornum = db_open( &d )) !=0 ) {   // open file
    printf( "Error opening file: " );
    switch( errornum ) {
      case DBE_OUT_OF_MEM:
        printf( "Not enough memory.\n" );
        break;
      case DBE_NO_FILE:
        printf( "Can not open file %s.\n", d.filename );
        break;
      case DBE_BAD_FORMAT:
        printf( "File %s is not a dBASE III file.\n", d.filename );
        break;
    }
    exit( 1 );
  }

  printf( "\nFile Name: %s",            d.filename );
  printf( "\nCurrent Record: %ld",      d.current_record );
  printf( "\nDBF signature: %d",        d.dbf_version );
  if ( d.status == db_not_updated ) printf( "\nStatus Not Updated" );
  else                              printf( "\nStatus Not Open" );
  printf( "\nNumber of Fields: %d",     d.num_fields );
  printf( "\nUpdate Year: %d",          d.update_yr );
  printf( "\nUpdate Month: %d",         d.update_mo );
  printf( "\nUpdate Day: %d",           d.update_day );
  printf( "\nNumber of Records: %ld",   d.records );
  printf( "\nRecord Length: %d",        d.record_length );
  for ( i=0; i<d.num_fields; i++ )
    printf( "\nField #%d: %s", i,       d.fields_ptr[i].name );
}

#endif


/****************************************************************************
*** end of DB_OPEN.C ***/